/*
 * phononbackend.cpp
 *
 *  Created on: 05.12.2010
 *      Author: darkstar
 */

#include <Phonon>

#include "phononbackend.h"
#include "debug.h"

/* PhononBackendAddOn */

static PhononBackendAddOn instance;

PhononBackendAddOn &PhononBackendAddOn::singleton()
{
    return instance;
}

AbstractPlaybackBackend *PhononBackendAddOn::createInstance()
{
    return new PhononBackend();
}

/* PhononBackend */

PhononBackend::PhononBackend(QObject *parent)
    :	AbstractPlaybackBackend(parent)
{
    DGUARDMETHODTIMED;
    DOP(debugPrintCaps());

    mediaObject_ = new Phonon::MediaObject(this);
    audioOutput_ = new Phonon::AudioOutput(Phonon::MusicCategory, this);
    Phonon::createPath(mediaObject_, audioOutput_);

    mediaObject_->setTransitionTime(0);
    mediaObject_->setTickInterval(500);

    connect(mediaObject_, SIGNAL(totalTimeChanged(qint64)), this, SLOT(mediaObjectTotalTimeChanged(qint64)));
    connect(mediaObject_, SIGNAL(stateChanged(Phonon::State, Phonon::State)), this, SLOT(mediaObjectStateChanged(Phonon::State, Phonon::State)));
    connect(mediaObject_, SIGNAL(tick(qint64)), this, SLOT(mediaObjectTick(qint64)));
    connect(mediaObject_, SIGNAL(finished()), this, SLOT(mediaObjectFinished()));
}

PhononBackend::~PhononBackend()
{

}

AbstractPlaybackBackend::PlayResult PhononBackend::play(Media *media)
{
    Phonon::MediaSource source(media->location());
    mediaObject_->setCurrentSource(source);

    if (mediaObject_->currentSource().type() == Phonon::MediaSource::Invalid)
        return AbstractPlaybackBackend::MediaNotSupported;

    mediaObject_->play();

    return AbstractPlaybackBackend::PlaybackInitiated;
}

void PhononBackend::mediaObjectTotalTimeChanged(qint64 totaltime)
{

}

void PhononBackend::mediaObjectStateChanged(Phonon::State newstate, Phonon::State oldstate)
{
    if (newstate == Phonon::PlayingState && newstate == Phonon::PausedState)
        emit resumed();
    else if (newstate == Phonon::PausedState && oldstate == Phonon::PlayingState)
        emit paused();
    else if (newstate == Phonon::LoadingState)
        emit started();
    else if (newstate == Phonon::ErrorState)
        emit errorMessage(mediaObject_->errorString());
}

void PhononBackend::mediaObjectTick(qint64 pos)
{
    emit currentPlayTime(pos / 1000);
}

void PhononBackend::mediaObjectFinished()
{
    emit eof();
}

bool PhononBackend::isRunning() const
{
    return true;
}

bool PhononBackend::isPlaying() const
{
    Phonon::State state = mediaObject_->state();
    return state == Phonon::PlayingState || state == Phonon::BufferingState || state == Phonon::PausedState;
}

bool PhononBackend::isPaused() const
{
    Phonon::State state = mediaObject_->state();
    return state == Phonon::PausedState;
}

void PhononBackend::pause()
{
    if (isPaused())
        mediaObject_->play();
    else
        mediaObject_->pause();
}

void PhononBackend::stop()
{
    mediaObject_->stop();
}

void PhononBackend::seekRelativeSeconds(int pos)
{
    mediaObject_->seek(mediaObject_->currentTime() + (pos * 1000));
}

void PhononBackend::seekAbsolutePercentage(int pos)
{
    mediaObject_->seek(mediaObject_->totalTime() * pos / 100);
}

void PhononBackend::seekAbsoluteSeconds(int pos)
{
    mediaObject_->seek(pos * 1000);
}

void PhononBackend::debugPrintCaps()
{
    DGUARDMETHODTIMED;

    DPRINTF("\nAvailable Audio Devices:");
    QList<Phonon::AudioOutputDevice> devices = Phonon::BackendCapabilities::availableAudioOutputDevices();
    for (int i = 0; i < devices.count(); ++i)
    {
        DPRINTF("  %s: %s", (const char *)devices[i].name().toUtf8(), (const char *)devices[i].description().toUtf8());
    }

    DPRINTF("\nSupported MIME Types:");

    QStringList mimeTypes = Phonon::BackendCapabilities::availableMimeTypes();
    foreach (QString mimeType, mimeTypes)
    {
        DPRINTF("  %s", (const char *)mimeType.toUtf8());
    }

    DPRINTF("\nAvailable Audio Effects:");
    QList<Phonon::EffectDescription> effects = Phonon::BackendCapabilities::availableAudioEffects();
    foreach (Phonon::EffectDescription effect, effects)
    {
        DPRINTF("  %s: %s", (const char *)effect.name().toUtf8(), (const char *)effect.description().toUtf8());

        Phonon::Effect *instance = new Phonon::Effect(effect, this);
        QList<Phonon::EffectParameter> parameters = instance->parameters();
        for (int i = 0; i < parameters.size(); ++i)
        {
            Phonon::EffectParameter parameter = parameters.at(i);

            QVariant defaultValue = parameter.defaultValue();
            QVariant minimumValue = parameter.minimumValue();
            QVariant maximumValue = parameter.maximumValue();

            QString valueString = QString("%1 / %2 / %3")
                    .arg(defaultValue.toString()).arg(minimumValue.toString())
                    .arg(maximumValue.toString());

            DPRINTF("    %s: %s (%s), %s", (const char *)parameter.name().toUtf8(), (const char *)parameter.description().toUtf8(), QVariant::typeToName(parameter.type()), (const char *)valueString.toUtf8());
        }
    }
}
