#include "playlistaspectoverviewmodel.h"

#include "debug.h"

enum Roles {
    NameRole = Qt::UserRole + 1,
    SelectedRole = Qt::UserRole + 2
};

/* PlayListAspectOverviewModel */

PlayListAspectOverviewModel::PlayListAspectOverviewModel(PlayListAspectOverviewFilter *aspect, const QString &roleName, OverviewFilterList *list)
:   QAbstractListModel(aspect),
    aspect_(aspect),
    roleName_(roleName),
    list_(list)
{
    DGUARDMETHOD;
    QHash<int, QByteArray> names;
    names[NameRole] = roleName.toLatin1();
    names[SelectedRole] = "selected";
    setRoleNames(names);
}

int PlayListAspectOverviewModel::rowCount(const QModelIndex &parent) const
{
    return list_->count();
}

int PlayListAspectOverviewModel::columnCount(const QModelIndex &parent) const
{
    return 1;
}

QVariant PlayListAspectOverviewModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (index.row() >= list_->size())
        return QVariant();

    OverviewFilterListEntry *entry = list_->at(index.row());

    switch (role)
    {
    case Qt::DisplayRole:
    {
        QVariant t;
        switch (index.column())
        {
        case 0:
            t = entry->text;
            break;
        case 1:
            t = entry->selected;
            break;
        }

        return t;
        break;
    }
    case NameRole:
        return entry->text;
        break;
    case SelectedRole:
        return entry->selected;
        break;
    default:
        return QVariant();
    }
}

QVariant PlayListAspectOverviewModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (orientation == Qt::Horizontal)
    {
        QString t;
        switch (section)
        {
        case 0:
            t = "Name";
            break;
        case 1:
            t = "Selected";
            break;
        }

        return t;
    }
    else
        return QVariant();
}

void PlayListAspectOverviewModel::sort(int column, Qt::SortOrder order)
{
    DGUARDMETHOD;
    /*
    if (column != aspect_->sortedColumn())
        aspect_->setSortParams(column, order == Qt::AscendingOrder);
    else
        aspect_->setSortParams(aspect_->sortedColumn(), !aspect_->isSortAscending());
    */

    DPRINTF("Header section %d clicked. Preparing playlist item index...", column);
}

void PlayListAspectOverviewModel::rereadData()
{
    emit layoutChanged();
}

/* PlayListAspectOverviewSelectionModel */

PlayListAspectOverviewSelectionModel::PlayListAspectOverviewSelectionModel(PlayListAspectOverviewModel *model)
    :   QItemSelectionModel(model),
        model_(model),
        updating_(false),
        selecting_(false)
{

}

void PlayListAspectOverviewSelectionModel::reset()
{
    DGUARDMETHOD;
    QItemSelectionModel::reset();
    transferSelectionToList();
}

void PlayListAspectOverviewSelectionModel::clear()
{
    DGUARDMETHOD;
    QItemSelectionModel::clear();
    transferSelectionToList();
}

void PlayListAspectOverviewSelectionModel::select(const QModelIndex &selection, QItemSelectionModel::SelectionFlags command)
{
    DGUARDMETHODF("command: %x", (int)command);
    QItemSelectionModel::select(selection, command);
    transferSelectionToList();
}

void PlayListAspectOverviewSelectionModel::select(const QItemSelection &selection, QItemSelectionModel::SelectionFlags command)
{
    DGUARDMETHODF("command: %x", (int)command);

    QItemSelectionModel::select(selection, command);

    if (selecting_ || updating_) return;

    selecting_ = true;

    QModelIndex firstIndex = model_->index(0);

    if (currentIndex() == firstIndex && isSelected(firstIndex) || !hasSelection())
        QItemSelectionModel::select(firstIndex, QItemSelectionModel::Clear | QItemSelectionModel::SelectCurrent);
    else
        QItemSelectionModel::select(firstIndex, QItemSelectionModel::Deselect);

    transferSelectionToList();

    selecting_ = false;
}

void PlayListAspectOverviewSelectionModel::transferSelectionToList()
{
    if (updating_) return;

    for (int i = 0; i < model_->list_->count(); ++i)
        model_->list_->at(i)->selected = isSelected(model_->index(i));

    emit listUpdateRequired();
}

void PlayListAspectOverviewSelectionModel::transferSelectionFromList()
{
    updating_ = true;

    QItemSelection selection;
    QItemSelection deselection;

    for (int i = 0; i < model_->list_->count(); ++i)
    {
        if (model_->list_->at(i)->selected)
            selection.append(QItemSelectionRange(model_->index(i)));
        else
            deselection.append(QItemSelectionRange(model_->index(i)));
    }

    select(selection, QItemSelectionModel::Select);
    select(deselection, QItemSelectionModel::Deselect);

    updating_ = false;
}
