/*
 * Copyright (C) 2006-2007	Andre Beckedorf
 * 					 		<evilJazz _AT_ katastrophos _DOT_ net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <qstringlist.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qapplication.h>

#include <qwidget.h>
#include <qpalette.h>

#include <QShowEvent>
#include <QKeyEvent>
#include <QEvent>
#include <QScrollBar>

#include "playlistoverview.h"
#include "playlistmanager.h"
#include "playlistview.h"
#include "configuration.h"
#include "mediadatabase.h"
#include "skin.h"

#include "debug.h"

PlayListOverview::PlayListOverview(QWidget *parent)
    : QWidget(parent),
    mediaDatabase_(NULL),
    aspectSource_(NULL)
{
    DGUARDMETHODTIMED;
    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->setMargin(0);
    layout->setSpacing(0);
    this->setLayout(layout);

    createColumnBox("Genres", genreBox_, genreHeading_, genreListBox_);
    genreListBox_->installEventFilter(this);
    layout->addLayout(genreBox_);

    // black line separator between list boxes...
    QWidget *dummywidget = new QWidget(this);
    dummywidget->setFixedWidth(1);
    QPalette palette;
    palette.setColor(QPalette::Window, QColor("#000000"));
    dummywidget->setPalette(palette);
    layout->addWidget(dummywidget);

    createColumnBox("Artists", artistBox_, artistHeading_, artistListBox_);
    artistListBox_->installEventFilter(this);
    layout->addLayout(artistBox_);

    // black line separator between list boxes...
    dummywidget = new QWidget(this);
    dummywidget->setFixedWidth(1);
    dummywidget->setPalette(palette);
    layout->addWidget(dummywidget);

    createColumnBox("Albums", albumBox_, albumHeading_, albumListBox_);
    albumListBox_->installEventFilter(this);
    layout->addLayout(albumBox_);

#ifdef QT4
    this->setFocusPolicy(Qt::StrongFocus);
#else
    this->setFocusPolicy(StrongFocus);
#endif
    this->setFocusProxy(genreListBox_);
}

PlayListOverview::~PlayListOverview()
{
}

void PlayListOverview::attachTo(PlayListAspectOverviewFilter *source)
{
    DGUARDMETHODTIMED;
    detach();

    if (source)
    {
        aspectSource_ = source;
        mediaDatabase_ = aspectSource_->playListAspect()->playList()->mediaDatabase();

        genreListBox_->setModel(aspectSource_->genreListModel());
        artistListBox_->setModel(aspectSource_->artistListModel());
        albumListBox_->setModel(aspectSource_->albumListModel());

        genreListBox_->setSelectionModel(aspectSource_->genreListSelectionModel());
        artistListBox_->setSelectionModel(aspectSource_->artistListSelectionModel());
        albumListBox_->setSelectionModel(aspectSource_->albumListSelectionModel());

        if (isVisible())
            aspectSource_->registerUser(this);
    }
}

void PlayListOverview::detach()
{
    DGUARDMETHODTIMED;
    if (aspectSource_)
    {
        albumListBox_->setModel(NULL);
        artistListBox_->setModel(NULL);
        genreListBox_->setModel(NULL);

        aspectSource_->unregisterUser(this);

        mediaDatabase_ = NULL;
        aspectSource_ = NULL;
    }
}

void PlayListOverview::createColumnBox(QString title, QVBoxLayout *&box, QLabel *&heading, QListView *&listbox)
{
    box = new QVBoxLayout(this);
    box->setMargin(0);
    box->setSpacing(0);

    heading = new QLabel(this);
#ifdef QT4
    heading->setAlignment(Qt::AlignCenter);
#else
    heading->setAlignment(Qt::AlignCenter | Qt::AlignVCenter | Qt::ExpandTabs);
#endif
    heading->setAutoFillBackground(true);
    heading->setText(title);

    listbox = new QListView(this);
    listbox->setFrameStyle(QFrame::NoFrame);
    listbox->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //listbox->setSelectionMode(QListView::Multi);

    box->addWidget(heading);
    box->addWidget(listbox);
}

void PlayListOverview::setSkinMode(const SkinModeInformation &modeInfo)
{
    if (modeInfo.playListOverviewFont().font.isUsed())
    {
        genreListBox_->setFont(modeInfo.playListOverviewFont().font);
        artistListBox_->setFont(modeInfo.playListOverviewFont().font);
        albumListBox_->setFont(modeInfo.playListOverviewFont().font);
    }
    else
    {
        genreListBox_->setFont(QFont());
        artistListBox_->setFont(QFont());
        albumListBox_->setFont(QFont());
    }

    // set general palette:
    QPalette palette = QApplication::palette();

    if (modeInfo.playListOverviewFont().fontColor.isUsed())
    {
        palette.setColor(QPalette::Text, modeInfo.playListOverviewFont().fontColor);
        palette.setColor(QPalette::Foreground, modeInfo.playListOverviewFont().fontColor);
    }

    if (modeInfo.playListOverviewBackgroundColorSelected().isUsed())
    {
        palette.setColor(QPalette::Highlight, modeInfo.playListOverviewBackgroundColorSelected());
        palette.setColor(QPalette::HighlightedText, modeInfo.playListOverviewFontColorSelected());
    }

    // set even and odd colors for items:
    if (modeInfo.playListOverviewBackgroundColor().isUsed())
        palette.setColor(QPalette::Base, modeInfo.playListOverviewBackgroundColor());

    genreListBox_->setPalette(palette);
    artistListBox_->setPalette(palette);
    albumListBox_->setPalette(palette);

    if (modeInfo.playListOverviewHeaderFont().font.isUsed())
    {
        genreHeading_->setFont(modeInfo.playListOverviewHeaderFont().font);
        artistHeading_->setFont(modeInfo.playListOverviewHeaderFont().font);
        albumHeading_->setFont(modeInfo.playListOverviewHeaderFont().font);
    }
    else
    {
        genreHeading_->setFont(QFont());
        artistHeading_->setFont(QFont());
        albumHeading_->setFont(QFont());
    }

    if (modeInfo.playListOverviewHeaderColor().isUsed())
    {
        QPalette palette;
        palette.setColor(QPalette::Background, modeInfo.playListOverviewHeaderColor());

        if (modeInfo.playListOverviewHeaderFont().fontColor.isUsed())
        {
            palette.setColor(QPalette::Text, modeInfo.playListOverviewHeaderFont().fontColor);
            palette.setColor(QPalette::ButtonText, modeInfo.playListOverviewHeaderFont().fontColor);
            palette.setColor(QPalette::Foreground, modeInfo.playListOverviewHeaderFont().fontColor);
        }

        genreHeading_->setPalette(palette);
        artistHeading_->setPalette(palette);
        albumHeading_->setPalette(palette);

        this->setPalette(palette);
    }
    else
    {
        genreHeading_->setPalette(QPalette());
        artistHeading_->setPalette(QPalette());
        albumHeading_->setPalette(QPalette());
    }

    if (modeInfo.playListOverviewScrollBarColor().isUsed())
    {
        genreListBox_->horizontalScrollBar()->setPalette(QPalette(modeInfo.playListOverviewScrollBarColor()));
        genreListBox_->verticalScrollBar()->setPalette(QPalette(modeInfo.playListOverviewScrollBarColor()));
        artistListBox_->horizontalScrollBar()->setPalette(QPalette(modeInfo.playListOverviewScrollBarColor()));
        artistListBox_->verticalScrollBar()->setPalette(QPalette(modeInfo.playListOverviewScrollBarColor()));
        albumListBox_->horizontalScrollBar()->setPalette(QPalette(modeInfo.playListOverviewScrollBarColor()));
        albumListBox_->verticalScrollBar()->setPalette(QPalette(modeInfo.playListOverviewScrollBarColor()));
    }
    else
    {
        genreListBox_->horizontalScrollBar()->setPalette(QPalette());
        genreListBox_->verticalScrollBar()->setPalette(QPalette());
        artistListBox_->horizontalScrollBar()->setPalette(QPalette());
        artistListBox_->verticalScrollBar()->setPalette(QPalette());
        albumListBox_->horizontalScrollBar()->setPalette(QPalette());
        albumListBox_->verticalScrollBar()->setPalette(QPalette());
    }
}

void PlayListOverview::setSelectionMode(QListView::SelectionMode mode)
{
    genreListBox_->setSelectionMode(mode);
    artistListBox_->setSelectionMode(mode);
    albumListBox_->setSelectionMode(mode);
}

void PlayListOverview::showEvent(QShowEvent *e)
{
    if (!e->spontaneous())
    {
        if (aspectSource_)
            aspectSource_->registerUser(this);
    }
}

void PlayListOverview::hideEvent(QHideEvent *e)
{
    if (!e->spontaneous())
    {
        if (aspectSource_)
            aspectSource_->unregisterUser(this);
    }
}

bool PlayListOverview::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::KeyPress)
    {
        QKeyEvent *e = static_cast<QKeyEvent *>(event);
        int key = e->key();

        QWidget *newFocusedWidget = NULL;

        if (object == genreListBox_)
        {
            if (key == Qt::Key_Right)
                newFocusedWidget = artistListBox_;
            else if (key == Qt::Key_Left)
                emit focusOutLeft();
        }
        else if (object == artistListBox_)
        {
            if (key == Qt::Key_Right)
                newFocusedWidget = albumListBox_;
            else if (key == Qt::Key_Left)
                newFocusedWidget = genreListBox_;
        }
        else if (object == albumListBox_)
        {
            if (key == Qt::Key_Right)
                emit focusOutRight();
            else if (key == Qt::Key_Left)
                newFocusedWidget = artistListBox_;
        }

        if (newFocusedWidget)
        {
            newFocusedWidget->setFocus();
            e->accept();
            return true;
        }

        return QWidget::eventFilter(object, event);
    }

    if (event->type() == QEvent::FocusIn)
        setFocusProxy(static_cast<QWidget *>(object));

    return QWidget::eventFilter(object, event);
}
